
%% MATLAB script for the rec.audio dithering demo
%% Copyright Christopher Hicks, 14th September 1995.

sample_rate = 22050;           % Hertz
total_time = 20;               % seconds

%% throughout 0dB is taken as the noise power generated by a 
%% quantiser of unity step size when presented with an 
%% uncorrelated random input.

A0 = +20;                      % initial amplitude +20dB 
A1 = -20;                      % final amplitude -20dB 

Ai = sqrt(10^(A0/10) / 12);    % convert dB to absolute amplitudes
Af = sqrt(10^(A1/10) / 12);

T = - total_time / log(Af/Ai); % calculate exponential decay constant 
N = total_time * sample_rate;  % calculate total length in samples

f0 = 500;                      % test tone frequency
t  = (0:(N-1))' / sample_rate; % make a time variable

%% calculate a decaying sinewave to double precision 
%% ie 64-bit floating point (48 bit mantissa)

disp('Calculating signal')
x = Ai * exp(-t/T) .* sin(2*pi*f0*t);

%% now for the fun bit...

dummy = zeros(2,1);                      % dummy variable for the noise shaper

disp('Calculating rounded output')
yq = round(x);                           % quantise, no dither
disp('Calculating dithered output')
yd = round(x + dither(2,N));             % quantise with LSB TPDF dither
disp('Calculating shaped output')
[yn, dummy] = nshp(x, 2, [1 0], dummy);  % quantise with a simple noise shaper 

wavwrite(yq*16+128, sample_rate, 'rounded.wav');
wavwrite(yd*16+128, sample_rate, 'dithered.wav');
wavwrite(yn*16+128, sample_rate, 'shaped.wav');

